@@ -1,6 +1,8 @@
 /* GLIB - Library of useful routines for C programming
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -32,7 +34,9 @@
 #include <glib/gtypes.h>
 #include <glib/gunicode.h>
 #include <glib/gbytes.h>
+#include <glib/gstrfuncs.h>
 #include <glib/gutils.h>  /* for G_CAN_INLINE */
+#include <string.h>
 
 G_BEGIN_DECLS
 
@@ -47,14 +51,31 @@ struct _GString
 
 GLIB_AVAILABLE_IN_ALL
 GString*     g_string_new               (const gchar     *init);
+GLIB_AVAILABLE_IN_2_78
+GString*     g_string_new_take          (gchar           *init);
 GLIB_AVAILABLE_IN_ALL
 GString*     g_string_new_len           (const gchar     *init,
                                          gssize           len);
 GLIB_AVAILABLE_IN_ALL
 GString*     g_string_sized_new         (gsize            dfl_size);
 GLIB_AVAILABLE_IN_ALL
-gchar*       g_string_free              (GString         *string,
+gchar*      (g_string_free)             (GString         *string,
                                          gboolean         free_segment);
+GLIB_AVAILABLE_IN_2_76
+gchar*       g_string_free_and_steal    (GString         *string) G_GNUC_WARN_UNUSED_RESULT;
+
+#if G_GNUC_CHECK_VERSION (2, 0) && (GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_76)
+
+#define g_string_free(str, free_segment)        \
+  (__builtin_constant_p (free_segment) ?        \
+    ((free_segment) ?                           \
+      (g_string_free) ((str), (free_segment)) : \
+      g_string_free_and_steal (str))            \
+    :                                           \
+    (g_string_free) ((str), (free_segment)))
+
+#endif /* G_GNUC_CHECK_VERSION (2, 0) && (GLIB_VERSION_MIN_REQUIRED >= GLIB_VERSION_2_76) */
+
 GLIB_AVAILABLE_IN_2_34
 GBytes*      g_string_free_to_bytes     (GString         *string);
 GLIB_AVAILABLE_IN_ALL
@@ -127,6 +148,11 @@ GLIB_AVAILABLE_IN_ALL
 GString*     g_string_erase             (GString         *string,
                                          gssize           pos,
                                          gssize           len);
+GLIB_AVAILABLE_IN_2_68
+guint         g_string_replace          (GString         *string,
+                                         const gchar     *find,
+                                         const gchar     *replace,
+                                         guint            limit);
 GLIB_AVAILABLE_IN_ALL
 GString*     g_string_ascii_down        (GString         *string);
 GLIB_AVAILABLE_IN_ALL
@@ -155,13 +181,22 @@ GString*     g_string_append_uri_escaped (GString         *string,
                                           const gchar     *reserved_chars_allowed,
                                           gboolean         allow_utf8);
 
-/* -- optimize g_strig_append_c --- */
 #ifdef G_CAN_INLINE
+
+#if defined (_MSC_VER) && !defined (__clang__)
+#pragma warning (push)
+#pragma warning (disable : 4141) /* silence "warning C4141: 'inline' used more than once" */
+#endif
+
+#ifndef __GTK_DOC_IGNORE__
+
+G_ALWAYS_INLINE
 static inline GString*
 g_string_append_c_inline (GString *gstring,
                           gchar    c)
 {
-  if (gstring->len + 1 < gstring->allocated_len)
+  if (G_LIKELY (gstring != NULL &&
+                gstring->len + 1 < gstring->allocated_len))
     {
       gstring->str[gstring->len++] = c;
       gstring->str[gstring->len] = 0;
@@ -170,19 +205,91 @@ g_string_append_c_inline (GString *gstring,
     g_string_insert_c (gstring, -1, c);
   return gstring;
 }
-#define g_string_append_c(gstr,c)       g_string_append_c_inline (gstr, c)
-#endif /* G_CAN_INLINE */
 
+#define g_string_append_c(gstr,c) \
+  g_string_append_c_inline (gstr, c)
+
+G_ALWAYS_INLINE
+static inline GString *
+g_string_append_len_inline (GString    *gstring,
+                            const char *val,
+                            gssize      len)
+{
+  gsize len_unsigned;
+
+  if G_UNLIKELY (gstring == NULL)
+    return g_string_append_len (gstring, val, len);
+
+  if G_UNLIKELY (val == NULL)
+    return (len != 0) ? g_string_append_len (gstring, val, len) : gstring;
+
+  if (len < 0)
+    len_unsigned = strlen (val);
+  else
+    len_unsigned = (gsize) len;
+
+  if (G_LIKELY (gstring->len + len_unsigned < gstring->allocated_len))
+    {
+      char *end = gstring->str + gstring->len;
+      if (G_LIKELY (val + len_unsigned <= end || val > end + len_unsigned))
+        memcpy (end, val, len_unsigned);
+      else
+        memmove (end, val, len_unsigned);
+      gstring->len += len_unsigned;
+      gstring->str[gstring->len] = 0;
+      return gstring;
+    }
+  else
+    return g_string_insert_len (gstring, -1, val, len);
+}
+
+#define g_string_append_len(gstr, val, len) \
+  g_string_append_len_inline (gstr, val, len)
+
+G_ALWAYS_INLINE
+static inline GString *
+g_string_truncate_inline (GString *gstring,
+                          gsize    len)
+{
+  gstring->len = MIN (len, gstring->len);
+  gstring->str[gstring->len] = '\0';
+  return gstring;
+}
+
+#define g_string_truncate(gstr, len) \
+  g_string_truncate_inline (gstr, len)
+
+#if G_GNUC_CHECK_VERSION (2, 0)
+
+#define g_string_append(gstr, val)                  \
+  (__builtin_constant_p (val) ?                     \
+    G_GNUC_EXTENSION ({                             \
+      const char * const __val = (val);             \
+      g_string_append_len (gstr, __val,             \
+        G_LIKELY (__val != NULL) ?                  \
+          (gssize) strlen (_G_STR_NONNULL (__val))  \
+        : (gssize) -1);                             \
+    })                                              \
+    :                                               \
+    g_string_append_len (gstr, val, (gssize) -1))
+
+#endif /* G_GNUC_CHECK_VERSION (2, 0) */
+
+#endif /* __GTK_DOC_IGNORE__ */
+
+#if defined (_MSC_VER) && !defined (__clang__)
+#pragma warning (pop) /* #pragma warning (disable : 4141) */
+#endif
+
+#endif /* G_CAN_INLINE */
 
 GLIB_DEPRECATED
 GString *g_string_down (GString *string);
 GLIB_DEPRECATED
 GString *g_string_up   (GString *string);
 
-#ifndef G_DISABLE_DEPRECATED
-#define  g_string_sprintf  g_string_printf
-#define  g_string_sprintfa g_string_append_printf
-#endif
+#define  g_string_sprintf  g_string_printf GLIB_DEPRECATED_MACRO_IN_2_26_FOR(g_string_printf)
+#define  g_string_sprintfa g_string_append_printf GLIB_DEPRECATED_MACRO_IN_2_26_FOR(g_string_append_printf)
 
 G_END_DECLS
 
